/***************************************************************************
 *   Copyright (C) 2007 by Sergey Semionov   *
 *   simon@vu.spb.ru   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LINUX_VEC535_DRIVER_H
#define __LINUX_VEC535_DRIVER_H

#include <linux/uaccess.h>
#include <linux/videodev.h>
#include <media/v4l2-common.h>
#include <media/v4l2-ioctl.h>
#include <linux/mutex.h>

#define VEC535_DRIVER_DEBUG	/* Turn on debug messages */

#ifdef VEC535_DRIVER_DEBUG
	#define PDEBUG(level, fmt, args...) \
			if (debug >= (level)) pr_info("[%s:%d] " fmt, \
									   		__FUNCTION__, __LINE__ , ## args)
#else
	#define PDEBUG(level, fmt, args...) do {} while(0)
#endif

/* An almost drop-in replacement for sleep_on_interruptible */
#define wait_interruptible(test, queue, wait) \
{ \
	add_wait_queue(queue, wait); \
	set_current_state(TASK_INTERRUPTIBLE); \
	if (test) \
		schedule(); \
	remove_wait_queue(queue, wait); \
	set_current_state(TASK_RUNNING); \
	if (signal_pending(current)) \
		break; \
}

/* number of queued bulk transfers to use, should be about 8 */
#define VEC535_NUMSBUF			8

/* number of frames which should stored until v4l asks for frame */
#define VEC535_NUMFRAMES		2
#define VEC535_NUMSCRATCH  		1024
/* size of usb transfers */
#define VEC535_PACKETSIZE	0x200
#define VEC535_BULK_IN_ADDR      2
#define VEC535_MAX_NULLPACKETS 		4000
#define VEC535_MAX_ERRORS 		200
#define VID_HARDWARE_VEC535		40
#define VEC535_NAME  "VEC535 USB Camera"

#define VEC535_5MPX_SIZE		5038848 //2592 x 1944		
struct usb_device;	

#define VEC535_MODES_AMMOUNT		3
const static int width_array[] = {864, 1280, 2136, 2592};
const static int height_array[] = {600, 960, 1602, 1944};
const static int disabled_array[] = {0, 0, 0, 1};
enum {
	MODE_864x600,
	MODE_1280x960,
	MODE_2136x1602,
	MODE_2592x1944
};

enum {
	FRAME_UNUSED,		/* Unused (no MCAPTURE) */
	FRAME_READY,		/* Ready to start grabbing */
	FRAME_GRABBING,		/* In the process of being grabbed into */
	FRAME_DONE,		/* Finished grabbing, but not been synced yet */
	FRAME_ERROR,		/* Something bad happened while processing */
};

enum {
	BUFFER_UNUSED,
	BUFFER_READY,
	BUFFER_BUSY,
	BUFFER_DONE,
};

struct vec535_sbuf {
	unsigned char *data;
};

struct vec535_scratch {
	unsigned char *data;
	volatile int state;
	int offset;
	int length;
};

struct vec535_frame {
	unsigned char *data;		/* Frame buffer */

	volatile int grabstate;	/* State of grabbing */

	unsigned char *prevlinepix;
	unsigned char *curpix;
	
};

struct usb_vec535 {
	struct video_device vdev;
	struct urb *urb[VEC535_NUMSBUF];
	struct usb_device *udev; //usb device
	struct usb_interface *interface; //usb interface
	int framecount;			//proceessed frames		
	int nullpackets;		//number of continiues null packets to reboot camera
	int dropped;			//number of dropped scratches
	int removed;			//flag that camera was removed
	int error;			//number of errors
	int streaming;			//Are we streaming currently
	int cwidth;				//camera width
	int cheight;				//camera height
	int fwidth;				//Frame width can be bigger that cwidth due to the cropping
	int fheight;				//Frame height 
	int user;  				//User count for exclusive use
	int curframe;				//current frame 0 or 1
	int bayeroffset;			//current offset for bayer (width * height max)!
		
	struct vec535_frame frame[VEC535_NUMFRAMES];
	struct vec535_scratch scratch[VEC535_NUMSCRATCH];
	struct vec535_sbuf sbuf[VEC535_NUMSBUF]; //buffers assigned to urb

	int sts_scratches_free;
	int sts_scratches_in_use;
	int scratch_next;			//scratch where result will be written over usb
	int scratch_use;			//scratch where will be next decode bayer
	struct mutex lock;
	wait_queue_head_t wq; //PROCESSES WAITING//
	char *fbuf;           //THE HOLE MEMORY

	//picture settings
	
	unsigned short hue;		//ok
	unsigned short brightness;
	unsigned short contrast;
	unsigned short gamma;		  //bgain&
	unsigned short whitebalance; //rgain
	unsigned short colour;		//saturation
	
	short Sx;
	short Sr;
	short Sg;
	short Sb;
	short exposure;
	short gain;		  //automatic or manual
	short palette; 
	short whiteness;
};



#endif /* __LINUX_VEC535_DRIVER */
